#ifndef __MidiShare__
#define __MidiShare__

#ifndef __WINDOWS_H
#include <windows.h>
#endif /* __WINDOWS_H */

/*****************************************************************************
 *                                MIDI SHARE									
 *----------------------------------------------------------------------------
 * MidiShare is a multi-tasking, real-time software environment, specially 
 * devised for the developing of MIDI applications with a triple target :
 * 
 * 	To propose solutions to currently met problems when developing 
 * 	 a real-time MIDI application : memory management, communication management, 
 *	 time management, task management. 
 * 
 * 	To enable the real-time and multi-tasking functioning of these applications, 
 * 	 i.e. to enable the sharing of all the necessary resources and their 
 * 	 simultaneous access.
 *
 * 	To make easier cooperation between independent MIDI applications by  proposing 
 * 	 a real-time mechanism of inter-application communications.
 * 
 * This file contains a complete description of all the MidiShare functions and 
 * procedures, as well as all the data structures in use.
 *----------------------------------------------------------------------------
 * 		            c GRAME 1989, 1990, 1991, 1992 
 *	          [Yann Orlarey, Herv Lequay, Dominique fober]						
 *----------------------------------------------------------------------------
 *
 *	version 1.20 
 *	version Lightspeed C version 3.01 G. Bloch Aout 1989 (modif YO 9-9-89)
 *	version Think C 4.0 : YO [13-11-89]
 *  extensions tches diffres : YO [05-12-89]
 *	version MPW C++ 3.1 : YO [09-03-90]
 *	harmonisation prototypes: YO [12-05-90]
 *	harmonisation MPW & TC: YO [12-05-90]
 *	implmentation de 'MidiShare()' sous forme de macro: YO [15-03-91]
 *  english version : YO [29-01-92]
 *  changes for TC5 : YO [03-02-93]
******************************************************************************/

/*******************************************************************************
 * MIDISHARE EVENTS
 *------------------------------------------------------------------------------
 * 	The listing below presents the different types of MidiShare handled events. 
 * 	This typology contains the whole of the standard Midi messages, plus specific 
 * 	messages such as typeNote corresponding to a note with its duration;  
 * 	or typeStream corresponding to a series of arbitrary bytes, possibly including 
 * 	data and status codes, sent directly without any processing; or typePrivate 
 * 	that are application private messages.
 * 	
 * 	All these codes may be used in the MidiNewEv function to allocate an event
 * 	of the desirable type and are accessible in an event evType field.
 *******************************************************************************/
			
#define typeNote		0 	/* note with pitch, velocity and duration		*/	
		
#define typeKeyOn		1 	/* Note On with pitch, velocity 				*/
#define typeKeyOff		2 	/* Note Off with pitch, velocity 				*/
#define typeKeyPress 		3 	/* Poly Key Pressure with pitch and pressure	*/
#define typeCtrlChange		4 	/* Control Change with controller ID and value	*/
#define typeProgChange		5 	/* Program Change with program ID number		*/
#define typeChanPress		6 	/* Channel Pressure with pressure value			*/
#define typePitchWheel		7 	/* Pitch Bend Change with LSB and MSB values	*/
		
#define typeSongPos		8 	/* Song Position Pointer with LSB and MSB values*/
#define typeSongSel		9 	/* Song Select with song ID number				*/
#define typeClock		10 	/* Timing Clock									*/
#define typeStart		11 	/* Start										*/
#define typeContinue		12 	/* Continue										*/
#define typeStop		13	/* Stop											*/
		
#define typeTune		14 	/* Tune Request									*/
#define typeActiveSens		15	/* Active Sensing								*/
#define typeReset		16	/* System Reset									*/
	
#define typeSysEx		17 	/* System Exclusive (only data bytes)			*/
#define typeStream		18 	/* arbitrary midi bytes (data and status codes)	*/
		
#define typePrivate		19	/*19..127 Private events for applications internal use*/
#define typeProcess		128	/* used by MidiShare for MidiCall and MidiTask	*/
#define typeDProcess		129	/* used by MidiShare for MidiDTask				*/
#define typeQuarterFrame	130	/* Midi time code quarter frame				*/

#define typeCtrl14b		131	
#define typeNonRegParam		132
#define typeRegParam		133

#define typeSeqNum		134	/* numro de squence					*/
#define typeText		135	/* vnement texte						*/
#define typeCopyright		136	/* message copyright					*/
#define typeSeqName		137	/* nom de squence ou de piste			*/
#define typeInstrName		138	/* nom d'instrument						*/
#define typeLyric		139	/* paroles dstines  etre chantes	*/
#define typeMarker		140	/* marqueur								*/
#define typeCuePoint		141	/* cue point							*/
#define typeChanPrefix		142	/* Midi Channel Prefix					*/
#define typeEndTrack		143	/* fin de piste							*/
#define typeTempo		144	/* changement de tempo					*/
#define typeSMPTEOffset		145	/* offset smpte							*/

#define typeTimeSign		146	/* indication de mesure					*/
#define typeKeySign		147	/* signature tonale						*/
#define typeSpecific		148	/* mta evt spcifique  un squenceur	*/

#define typeReserved		149	/*149..254	reserved for future extensions		*/
		
#define typeDead		255	/* dead Task or DTask							*/
		


/******************************************************************************
* MIDI STATUS CODE								
*******************************************************************************/

#define NoteOff		0x80	
#define NoteOn		0x90
#define PolyTouch	0xa0
#define ControlChg	0xb0
#define ProgramChg	0xc0
#define AfterTouch	0xd0
#define PitchBend	0xe0
#define SysRealTime	0xf0
#define SysEx		0xf0
#define QFrame		0xf1
#define SongPos		0xf2
#define SongSel		0xf3
#define UnDef2		0xf4
#define UnDef3		0xf5
#define Tune		0xf6
#define EndSysX		0xf7
#define MClock		0xf8
#define UnDef4		0xf9
#define MStart		0xfa
#define MCont		0xfb
#define MStop		0xfc
#define UnDef5		0xfd
#define ActSense	0xfe
#define MReset		0xff


/*******************************************************************************
* SERIAL PORTS									
*------------------------------------------------------------------------------
* The Modem and printer serial ports ID numbers.							
*******************************************************************************/
			
#define ModemPort	0
#define PrinterPort	1
		

/******************************************************************************
* ERROR CODES								
*------------------------------------------------------------------------------
* List of the error codes returned by some MidiShare functions.
*******************************************************************************/
		
#define MIDIerrSpace 	-1		/* plus de place dans la freeList */
#define MIDIerrRefNum	-2		/* mauvais numero de reference */
#define MIDIerrBadType	-3		/* mauvais type d'evnement */
#define MIDIerrIndex	-4		/* mauvais index d'acces  un vnement */
#define MIDIerrEv 	-5		/* evt pass en argument = nil 		*/
#define MIDIerrUndef 	-6		/* evt pass en argument indfini 	*/
		

/******************************************************************************
* SYNCHRONISATION CODES
*------------------------------------------------------------------------------
* List of the error codes returned by some MidiShare functions.																	
*******************************************************************************/
		
#define MIDISyncExternal 	0x8000		/* bit-15 for external synchronisation */
#define MIDISyncAnyPort		0x4000		/* bit-14 for synchronisation on any port */


/******************************************************************************
* CHANGE CODES							
*------------------------------------------------------------------------------
When an application need to know about context modifications like opening and
closing of applications, opening and closing of midi ports, changes in 
connections between applications, it can install an ApplAlarm (with 
MidiSetApplAlarm). This ApplAlarm is then called by MidiShare every time a 
context modification happens with a 32-bits code describing the modification. 
The hi 16-bits part of this code is the refNum of the application involved in 
the context modification, the low 16-bits part describe the type of change as
listed here.
*******************************************************************************/
		
enum{	MIDIOpenAppl=1,
        MIDICloseAppl,
	MIDIChgName,
	MIDIChgConnect,
	MIDISyncStart,
	MIDISyncStop,
	MIDIChangeSync,
	MIDIOpenModem=100,
	MIDICloseModem,
	MIDIOpenPrinter,
	MIDIClosePrinter
};

enum { false, true };

typedef unsigned char Byte;
typedef char FAR *Ptr;
typedef unsigned char Boolean;

		
/******************************************************************************
* 							    EVENTS STRUCTURES							
*------------------------------------------------------------------------------
* Tous les vnements sont construits  l'aide d'une ou plusieurs cellules de	
* taille fixe (16 octets). La plupart ncessitent une seule cellule. D'autres	
* comme les System Exclusive ncessitent plusieurs cellules chanes.			
*******************************************************************************/


/*------------------------ System Exclusive extension cell ----------------------*/

	typedef struct TMidiSEX FAR *MidiSEXPtr;
	typedef struct TMidiSEX
	{
		MidiSEXPtr link;					/* link to next cell				*/
		Byte data[12];						/* 12 data bytes					*/
	}	TMidiSEX;						
	

/*------------------------------ Private extension cell -------------------------*/

	typedef struct TMidiST FAR *MidiSTPtr;
	typedef struct TMidiST					
	{
		Ptr ptr1; /* 4 32-bits fields */ 				
		Ptr ptr2;							
		Ptr ptr3;
		Ptr ptr4;						
	}	TMidiST;						
		

/*------------------------- Common Event Structure ----------------------*/

	typedef struct TMidiEv FAR *MidiEvPtr;
	typedef struct TMidiEv
	{
		MidiEvPtr	link;		/* link to next event			*/
		unsigned long	date;		/* event date (in ms)	       		*/
		Byte		evType;		/* event type		 		*/
		Byte		refNum;		/* sender reference number	 	*/
		Byte		port;		/* Midi port 				*/
		Byte		chan;		/* Midi channel		   		*/
		union {				/* info depending of event type :	*/
			struct {			/* for notes			*/
				Byte pitch;		/* pitch  			*/
				Byte vel;		/* velocity 			*/
				unsigned short dur;	/* duration 			*/
			} note;

			struct {			/* for MidiFile time signature  */
				Byte numerator;		/* numerator			*/
				Byte denominator;	/* denominator as neg power	*/
							/* of 2. (2= quarter note)	*/
				Byte nClocks;		/* number of Midi clocks in	*/
							/* a metronome click		*/
				Byte n32nd;		/* number of 32nd notes in	*/
							/* a Midi quarter note		*/
			} timeSign;

			struct {			/* for MidiFile key signature	*/
				char ton;		/* 0: key of C, 1: 1 sharp	*/
							/* -1: 1 flat etc...		*/
				Byte mode;		/* 0: major 1: minor		*/
				Byte unused[2];

			} keySign;

			struct {			/* for paramchg & 14-bits ctrl	*/
				short num;		/* param or ctrl num		*/
				short val;		/* 14-bits value		*/
			} param;

			struct {			/* for MidiFile sequence number */
				unsigned short number;
				short unused;
			} seqNum;
			long tempo;			/* MidiFile tempo in		*/
							/* microsec/Midi quarter note	*/
			Byte data[4];			/* for other small events   	*/
			MidiSEXPtr linkSE;		/* link to last sysex extension	*/
			MidiSTPtr linkST;		/* link to private extension	*/
		} info;
	} TMidiEv;		

	
/*------------------------------ sequence header ---------------------------*/

	typedef struct TMidiSeq FAR *MidiSeqPtr;
	typedef struct TMidiSeq
	{
		MidiEvPtr first;					/* first event pointer 				*/
		MidiEvPtr last;						/* last event pointer 				*/
		Ptr undef1;
		Ptr undef2;
	}	TMidiSeq;
	

/*-------------------------------- input Filter -------------------------------*/

	typedef struct TFilter FAR *FilterPtr;
	typedef struct TFilter
	{
		char port[32];						/* 256 bits	*/
		char evType[32];					/* 256 bits */
		char channel[2];					/*  16 bits	*/
		char unused[2];						/*  16 bits */
	} TFilter; 
	

/*------------------------ MidiShare application name ------------------------*/

	typedef char FAR *MidiName;
	

/*------------------------ Synchronisation informations -----------------------*/

	typedef struct TSyncInfo FAR *SyncInfoPtr;
	typedef struct TSyncInfo
	{
 		long		time;
 		long		reenter;
 		unsigned short	syncMode;
 		Byte		syncLocked; 
 		Byte		syncPort;
		long		syncStart;
		long		syncStop;
		long		syncOffset;
		long		syncSpeed;
		long		syncBreaks;
		short		syncFormat;
	} TSyncInfo; 

	typedef struct TSmpteLocation FAR *SmpteLocPtr;
	typedef struct TSmpteLocation
	{
 		short		format;			/* (0:24f/s, 1:25f/s, 2:30DFf/s, 3:30f/s)	*/
 		short		hours;			/* 0..23							*/
 		short		minutes;		/* 0..59							*/
 		short		seconds;		/* 0..59							*/
 		short		frames;			/* 0..30 (according to format)		*/
 		short		fracs;			/* 0..99 (1/100 of frames)			*/
	} TSmpteLocation; 
	

/*--------------------- Statistic description of MidiShare --------------------*/
typedef struct MidiStat FAR *MidiStatPtr;
typedef struct MidiStat 
{
	unsigned long rcvErrs;				/* number of receive errors 	*/
	unsigned long allocErrs;			/* number of allocation errors 	*/
	unsigned long rcvEvs;				/* number of rcv events 		*/
	unsigned long xmtEvs;				/* number of xmt events 		*/
} MidiStat;


/*----------------------------- Alarms prototypes ----------------------------*/

typedef void (WINAPI _export *TaskPtr)		( unsigned long date, short refNum, long a1,long a2,long a3 );
typedef void (WINAPI _export *RcvAlarmPtr)	( short refNum );
typedef void (WINAPI _export *ApplAlarmPtr)	( short refNum, long code );
typedef void (WINAPI _export *ApplyProcPtr)	( MidiEvPtr e );


/******************************************************************************
* 							   FIELD MACROS							
*------------------------------------------------------------------------------
* Somes macros to read and write event's fields					 
*******************************************************************************/

	#define Link(e) 	( (e)->link )
	#define Date(e) 	( (e)->date )
	#define EvType(e) 	( (e)->evType )
	#define RefNum(e) 	( (e)->refNum )	
	#define Port(e) 	( (e)->port )
	#define Canal(e)	( (e)->chan )
	#define Chan(e)		( (e)->chan )
	#define Pitch(e) 	( (e)->info.note.pitch ) 
	#define Vel(e) 		( (e)->info.note.vel ) 
	#define Dur(e) 		( (e)->info.note.dur )
	#define Data(e) 	( (e)->info.data )
	#define LinkSE(e) 	( (e)->info.linkSE )
	#define LinkST(e) 	( (e)->info.linkST )

	#define TSNum(e)	( (e)->info.timeSign.numerator )
	#define TSDenom(e)	( (e)->info.timeSign.denominator )
	#define TSClocks(e)	( (e)->info.timeSign.nClocks )
	#define TS32nd(e)	( (e)->info.timeSign.n32nd )

	#define KSTon(e)	( (e)->info.keySign.ton )
	#define KSMode(e)	( (e)->info.keySign.mode )

	#define Tempo(e)	( (e)->info.tempo )
	#define SeqNum(e)	( (e)->info.seqNum.number )
	#define ChanPrefix(e)	( (e)->info.data[0] )

	#define First(e)	( (e)->first )
	#define Last(e)		( (e)->last )
	#define FirstEv(e)	( (e)->first )
	#define LastEv(e)	( (e)->last )

/******************************************************************************
* 							  FILTER MACROS							
*------------------------------------------------------------------------------
* somes macros to set and reset filter's bits. 
*******************************************************************************/

#ifdef __cplusplus
inline void 	AcceptBit( char FAR *a, Byte n) 	{ (a)[(n)>>3] |=  (1<<((n)&7)); }
inline void 	RejectBit( char FAR *a, Byte n) 	{ (a)[(n)>>3] &= ~(1<<((n)&7)); }
inline void 	InvertBit( char FAR *a, Byte n) 	{ (a)[(n)>>3] ^=  (1<<((n)&7)); }

inline Boolean 	IsAcceptedBit( char FAR *a, Byte n) 	{ return (a)[(n)>>3] & (1<<((n)&7)); }

#else

	#define AcceptBit(a,n)		( ((char FAR*) (a))[(n)>>3] |=   (1<<((n)&7)) )
	#define RejectBit(a,n)		( ((char FAR*) (a))[(n)>>3] &=  ~(1<<((n)&7)) )
	#define InvertBit(a,n)		( ((char FAR*) (a))[(n)>>3] ^=   (1<<((n)&7)) )

	#define IsAcceptedBit(a,n)	( ((char FAR*) (a))[(n)>>3]  &   (1<<((n)&7)) )
#endif /* __cplusplus */


/*******************************************************************************
* 								ENTRY POINTS									
*------------------------------------------------------------------------------
* All the MidiShare entry point are implemented as below :
*			moveq	#nn, d0	;move entry point ID number into reg d0													
*			move.l	$B8, a0	;read address of MS dispatch routine from vector $B8													
*			jmp		(a0)	;jump to dispatch,													
*																				
* Vector $B8 is initialy set by MidiShare Init at boot time.		
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
 
/*--------------------------- Global MidiShare environment --------------------*/

  void  WINAPI _export MidiShareSpecialInit(unsigned long defaultSpace);
  short WINAPI _export MidiGetVersion(void);

  short WINAPI _export MidiCountAppls(void);
  short WINAPI _export MidiGetIndAppl(short index);
  short WINAPI _export MidiGetNamedAppl(MidiName name);
 
/*----------------------------- SMPTE synchronization -------------------------*/

  void WINAPI _export MidiGetSyncInfo(SyncInfoPtr p);
  void WINAPI _export MidiSetSyncMode(unsigned short mode);
  long WINAPI _export MidiGetExtTime(void);
  long WINAPI _export MidiInt2ExtTime(long xl);
  long WINAPI _export MidiExt2IntTime(long xl);
  void WINAPI _export MidiTime2Smpte(long time, short format, SmpteLocPtr loc);
  long WINAPI _export MidiSmpte2Time(SmpteLocPtr loc);

  LPDWORD WINAPI _export MidiGetTimeAddr(void);

/*----------------------------- Open / close application ----------------------*/

  short WINAPI _export MidiOpen(MidiName applName);
  void  WINAPI _export MidiClose(short refNum);


/*--------------------------- Application configuration -----------------------*/

  MidiName	WINAPI _export MidiGetName(short refNum);
  void		WINAPI _export MidiSetName(short refNum, MidiName applName);
  LPVOID	WINAPI _export MidiGetInfo(short refNum);
  void		WINAPI _export MidiSetInfo(short refNum,LPVOID InfoZone);
  FilterPtr	WINAPI _export MidiGetFilter(short refNum);
  void		WINAPI _export MidiSetFilter(short refNum, FilterPtr f);
  RcvAlarmPtr	WINAPI _export MidiGetRcvAlarm(short refNum);
  void		WINAPI _export MidiSetRcvAlarm(short refNum, RcvAlarmPtr alarm);
  ApplAlarmPtr	WINAPI _export MidiGetApplAlarm(short refNum);
  void		WINAPI _export MidiSetApplAlarm(short refNum, ApplAlarmPtr alarm);


/*------------------------- Inter-Application Connections ---------------------*/

  void    WINAPI _export MidiConnect(short src, short dest , Boolean state);
  Boolean WINAPI _export MidiIsConnected(short src, short dest);


/*-------------------------------- Serial ports -------------------------------*/

  Boolean WINAPI _export MidiGetPortState(short port);
  void    WINAPI _export MidiSetPortState(short port, Boolean state);


/*-------------------------- Events and memory managing -----------------------*/

  unsigned long	WINAPI _export MidiFreeSpace(void);

  MidiEvPtr	WINAPI _export MidiNewCell(void);
  void		WINAPI _export MidiFreeCell(MidiEvPtr e);
  unsigned long	WINAPI _export MidiTotalSpace(void);
  unsigned long	WINAPI _export MidiGrowSpace(long n);

  MidiEvPtr	WINAPI _export MidiNewEv(short typeNum);
  MidiEvPtr	WINAPI _export MidiCopyEv(MidiEvPtr e);
  void		WINAPI _export MidiFreeEv(MidiEvPtr e);

  void		WINAPI _export OldMidiSetField(MidiEvPtr e, short f, long v);
  long		WINAPI _export OldMidiGetField(MidiEvPtr e, short f);
  short		WINAPI _export OldMidiCountFields(MidiEvPtr e);

  void		WINAPI _export MidiSetField(MidiEvPtr e, long f, long v);
  long		WINAPI _export MidiGetField(MidiEvPtr e, long f);
  void		WINAPI _export MidiAddField(MidiEvPtr e, long v);
  long		WINAPI _export MidiCountFields(MidiEvPtr e);


/*------------------------------- Sequence managing ---------------------------*/

  MidiSeqPtr WINAPI _export MidiNewSeq(void);
  void       WINAPI _export MidiAddSeq(MidiSeqPtr s, MidiEvPtr e);
  void       WINAPI _export MidiFreeSeq(MidiSeqPtr s);
  void       WINAPI _export MidiClearSeq(MidiSeqPtr s);
  void       WINAPI _export MidiApplySeq(MidiSeqPtr s, ApplyProcPtr proc);


/*------------------------------------- Time ----------------------------------*/

  unsigned long WINAPI _export MidiGetTime(void);


/*------------------------------------ Sending --------------------------------*/
  void WINAPI _export MidiSendIm(short refNum, MidiEvPtr e);
  void WINAPI _export MidiSend(short refNum, MidiEvPtr e);
  void WINAPI _export MidiSendAt(short refNum, MidiEvPtr e, unsigned long d);


/*------------------------------------ Receving -------------------------------*/
  unsigned long	WINAPI _export MidiCountEvs(short refNum);
  MidiEvPtr	WINAPI _export MidiGetEv(short refNum);
  MidiEvPtr	WINAPI _export MidiAvailEv(short refNum);
  void		WINAPI _export MidiFlushEvs(short refNum);


/*----------------------------------- Mail boxes ------------------------------*/

  LPVOID WINAPI _export MidiReadSync(LPVOID FAR *adrMem);
  LPVOID WINAPI _export MidiWriteSync(LPVOID FAR *adrMem,LPVOID val);


/*---------------------------------- Task Managing ----------------------------*/

  void      WINAPI _export MidiCall (TaskPtr routine, unsigned long date, short refNum, long a1,long a2,long a3);

  MidiEvPtr WINAPI _export MidiTask(TaskPtr routine, unsigned long date, short refNum, long a1,long a2,long a3);
  MidiEvPtr WINAPI _export MidiDTask(TaskPtr routine, unsigned long date, short refNum, long a1,long a2,long a3);

  void      WINAPI _export MidiForgetTask(MidiEvPtr FAR *e);

  long      WINAPI _export MidiCountDTasks(short refNum);

  void      WINAPI _export MidiFlushDTasks(short refNum);

  void      WINAPI _export MidiExec1DTask(short refNum);

/*----------------------------------- MidiShare -------------------------------*/

  Boolean WINAPI _export MidiShare(void);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __MidiShare__ */
