/* ============================================================================

MidiShare Application Skeleton

===============================================================================
MidiShare is a real-time multi-tasking MIDI operating system
specially devised for the development of musical applications.

SKELETON is a sample of MidiShare client application. It can be freely
used to develop new applications. By default, this application resend
all the received events, without any modification.

For your purpose, you can modify the following functions and constants :

- the constant ApplName : the MidiShare name of the application

- the function ReceiveEvents : handle all the incoming events. Setup to
  make a simple Midi Thru.

- The constant wantRealTime : when 1, ReceiveEvents is called at interrupt
  level and therefore, it can just used a restricted set of the system API
  (see your developper documentation). when 0, ReceiveEvents is called in
  the main loop and can access to all the functions of the system.

- The constant wantFilter : when 1, the application install a filter to
  select its incoming events, when 0, there is no filter and the application
  receive all the events.

- The function InstallFilter : it configure the input filter

===============================================================================
c GRAME 1987-95
============================================================================ */

#ifndef __MidiShare__
#include "mshare.h"
#endif

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifndef __SKELETON__
#include "skeleton.h"
#endif


// ----------------------------------------------------------------------------
// CONSTANTS
// ----------------------------------------------------------------------------
#define wantFilter	0
#define wantRealTime	1


// ----------------------------------------------------------------------------
// GLOBAL VARIABLES
// ----------------------------------------------------------------------------
HINSTANCE	ghInst;
HWND			ghWnd;

short myRefNum;		// reference number of the MidiShare application
TFilter myFilter;		// input filter
MidiName ApplName = "Skeleton";


// ----------------------------------------------------------------------------
// FUNCTIONS PROTOTYPES
// ----------------------------------------------------------------------------
LONG FAR PASCAL _export ApplWindProc(HWND,UINT,UINT,LONG);
void FAR PASCAL _export ReceiveEvents(register short);

BOOL NEAR SetupWindow(HINSTANCE, int);

VOID NEAR PASCAL Create(HWND);
VOID NEAR PASCAL DoRedraw(HWND);
void NEAR PASCAL DoCommand(HWND,UINT,LONG);
VOID NEAR PASCAL DestroyIt(HWND);

VOID NEAR DoQuitCmd(HWND);

void NEAR AlertUser(char *msg);
BOOL NEAR SetUpMidi(void);
void NEAR InstallFilter(void);
void NEAR DoIdle(void);
void About(HINSTANCE hInst,HWND hWnd);


/* ----------------------------------------------------------------------------

APPLICATION MAIN ENTRY POINT

---------------------------------------------------------------------------- */
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
						 LPSTR lpszCmdLine, int nCmdShow)
{
  MSG msg;

  if ( hPrevInstance ) return FALSE ;
  ghInst = hInstance;
  if (!SetupWindow(hInstance, nCmdShow)) return FALSE;
  if (!SetUpMidi()) return FALSE;
  while ( TRUE ) {
	 if ( PeekMessage(&msg,(HWND)NULL,0,0,PM_REMOVE) ) {
		if ( msg.message == WM_QUIT ) break;
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	 }
	 else DoIdle();
  }
  MidiClose(myRefNum);
  return msg.wParam;
}


/* ----------------------------------------------------------------------------

       Window Setup

---------------------------------------------------------------------------- */
BOOL NEAR SetupWindow (HINSTANCE hInst, int cmd)
{
	WNDCLASS wndclass;

	wndclass.style = CS_BYTEALIGNCLIENT|CS_VREDRAW|CS_HREDRAW;
	wndclass.lpfnWndProc = ApplWindProc;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;
	wndclass.hInstance = hInst;
	wndclass.hIcon = LoadIcon(hInst,"IconSkeleton");
	wndclass.hCursor = LoadCursor(NULL,IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wndclass.lpszMenuName = MAKEINTATOM(IDD_MENU);
	wndclass.lpszClassName = ApplName;

	if ( !RegisterClass(&wndclass) ) return FALSE;
	ghWnd = CreateWindow(ApplName,	// class name
				 ApplName,					// title
				 WS_OVERLAPPEDWINDOW,	// style
				 CW_USEDEFAULT,			// horizontal position
				 CW_USEDEFAULT,			// vertical position
				 CW_USEDEFAULT,		// width
				 CW_USEDEFAULT,		// height
				 (HWND)NULL,			// parent window
				 (HMENU)NULL,			// menu class
				 (HINSTANCE)hInst,	// window instance
				 (LPSTR)NULL			// 
				);
	if (ghWnd) {
		ShowWindow(ghWnd, cmd);
		UpdateWindow(ghWnd);
		return TRUE;
	}
	return FALSE;
}


/* ----------------------------------------------------------------------------

Main Window Proc

---------------------------------------------------------------------------- */
LONG FAR PASCAL _export ApplWindProc(HWND hWnd,UINT wMessage,UINT wParam,LONG lParam)
{
  switch ( wMessage ) {
	 case WM_CREATE :
		Create(hWnd);
		break;
	 case WM_PAINT :
		DoRedraw(hWnd);
		break;
	 case WM_COMMAND :
		DoCommand(hWnd,wParam,lParam);
		break;
	 case WM_DESTROY :
		DestroyIt(hWnd);
		break;
	 default :
		return DefWindowProc(hWnd,wMessage,wParam,lParam);
  }
  return 0L;
}


/* ----------------------------------------------------------------------------

MESSAGE : W M _ C R E A T E

---------------------------------------------------------------------------- */
void NEAR PASCAL Create(HWND hWnd)
{
  /*
		your code here
  */
}


/* ----------------------------------------------------------------------------

MESSAGE : W M _ P A I N T

---------------------------------------------------------------------------- */
void NEAR PASCAL DoRedraw(HWND hWnd)
{
  PAINTSTRUCT ps;

  BeginPaint(hWnd,&ps);
  /*
		your window redraw code here
  */
  EndPaint(hWnd,&ps);
}


/* ----------------------------------------------------------------------------

MESSAGE : W M _ C O M M A N D

---------------------------------------------------------------------------- */
void NEAR PASCAL DoCommand(HWND hWnd, UINT wParam, LONG lParam)
{
  switch ( wParam ) {
	 case IDM_QUIT :
		DoQuitCmd(hWnd);
		break;
	 case IDM_ABOUT :
		About(ghInst, ghWnd);
		break;
  }
}


/* ----------------------------------------------------------------------------

Displaying infos about the application

---------------------------------------------------------------------------- */
int FAR PASCAL _export AboutDlgProc(HWND hDlg,UINT msg,WPARAM wParam,LPARAM lParam)
{
  short version;
  char buff[10];

  switch (msg) {
	 case WM_INITDIALOG:
		version= MidiGetVersion();
		wsprintf (buff, "%d. %d", version/100, version%100);
		SetDlgItemText (hDlg, kVersion, buff);
		return TRUE;

	 case WM_COMMAND:
		switch (wParam) {
			case IDC_OK :
				EndDialog(hDlg, TRUE);
				break;
		}
      break;
  }
  return FALSE;
}

void About(HINSTANCE hInst,HWND hWnd)
{
  DLGPROC dlgProc;

  dlgProc = (DLGPROC)MakeProcInstance((FARPROC)AboutDlgProc,hInst);
  DialogBox(hInst,"About", hWnd, dlgProc);
  FreeProcInstance(dlgProc);
}

/* ----------------------------------------------------------------------------

MESSAGE : W M _ D E S T R O Y

---------------------------------------------------------------------------- */
VOID NEAR PASCAL DestroyIt(HWND hWnd)
{
  PostQuitMessage(0);
}


/* ----------------------------------------------------------------------------

WM_COMMAND : IDM_QUIT

---------------------------------------------------------------------------- */
void NEAR DoQuitCmd(HWND hWnd)
{
  int rep;

  rep = MessageBox(hWnd,"\n\nReally quit Skeleton ?",ApplName,MB_OKCANCEL|MB_ICONINFORMATION);
  switch ( rep ) {
	 case IDOK :
		SendMessage(hWnd,WM_SYSCOMMAND,SC_CLOSE,0L);
		break;
	 case IDCANCEL :
		break;
  }
}


/* ----------------------------------------------------------------------------

Alert user : for initialization failures

---------------------------------------------------------------------------- */
void NEAR AlertUser(char *msg)
{
  MessageBox (ghWnd,msg,ApplName,MB_OK|MB_ICONSTOP);
  SendMessage(ghWnd,WM_SYSCOMMAND,SC_CLOSE,0L);
}


/* ----------------------------------------------------------------------------

MIDISHARE : opening a client application

---------------------------------------------------------------------------- */
BOOL NEAR SetUpMidi()
{
  // Check for MidiShare (not needed, just intended for compatibility
  // with other platforms
  if ( !MidiShare() ) {
	 AlertUser("Can't access to 'MSHARE.DLL'");
	 return FALSE;
  }
  // Open MidiShare client application
  myRefNum = MidiOpen(ApplName);
  if ( myRefNum == MIDIerrSpace ) {
	 AlertUser("Too much MidiShare client applications");
	 return FALSE;
  }

  if ( wantRealTime ) {
	 // install a receive alarm, called by MidiShare at interrupt level
	 // each time that there are incoming events for the aplication
	 MidiSetRcvAlarm (myRefNum,ReceiveEvents);
  }
  // Install a filter for incoming events
  if ( wantFilter ) InstallFilter();

  // Connect the application to the input / output ports
  MidiConnect(0,myRefNum,true);	// input
  MidiConnect(myRefNum,0,true);	// output
  return TRUE;
}


/* ----------------------------------------------------------------------------

MIDISHARE : handling incoming events

---------------------------------------------------------------------------- */
void FAR PASCAL _export ReceiveEvents(register short r)
{
	register MidiEvPtr e;

	while (e = MidiGetEv(r)) {	// for each event
		/*
			your code here
		*/
		MidiSend (r,e);			// resend the event
	}
}

/* ----------------------------------------------------------------------------

APPLICATION FILTER INSTALLATION

-------------------------------------------------------------------------------

This function define the application filter. A filter consists of 3 parts,
each one is a boolean table :
	a 256 bits boolean table for the accepted ports
	a 256 bits boolean table for the accepted events types
	a 16 bits boolean table for the accepted MIDI channels
The following filter is configured to accept everything on all the ports
and all tthe MIDI channels.

---------------------------------------------------------------------------- */
void NEAR InstallFilter()
{
  short i;

  // accept all event types from all the input ports
  for (i=0;i<256;i++) {
	 AcceptBit(myFilter.evType,i);
	 AcceptBit(myFilter.port,i);
  }
  // and from all the MIDI channels
  for (i=0;i<16;i++) AcceptBit(myFilter.channel,i);
  // Install the filter
  MidiSetFilter(myRefNum,&myFilter);
}

/* --------------------------------------------------------------------------*/
void NEAR DoIdle()
{
  register long n;
  register short r;

  r = myRefNum;
  if ( !wantRealTime ) ReceiveEvents(r);
  // process all the application defered tasks
  for (n=MidiCountDTasks(r);n>0;--n) MidiExec1DTask(r);
}
